"""
============================
Author:柠檬班-木森
Time:2021/2/27 19:44
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
=======
"""

"""
python setup.py sdist
python setup.py sdist bdist_wheel
twine upload dist/*

"""



"""
============================
Project: Apin
Author:柠檬班-木森
Time:2021/8/11 20:25
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
Site: http://www.lemonban.com
Forum: http://testingpai.com 
============================
"""
from apin.core.tools import *
from hooks.db_check_hook import *
from hooks.fixture_hook import *


def rand_phone(st1):
    return random_mobile()


def get_sign(a,b):
    return get_timestamp()



"""
============================
Author:柠檬班-木森
Time:2021/3/1 20:20
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
=======
"""
from apin.core.httptest import HttpCase


class Testbaidu(HttpCase):
    name ='百度测试'
    host = "http://www.baidu.com"

    Cases = [
        # 用例1：普通用户注册
        {
            'title': "百度",
            'interface': "/",
            "method": "get",
        },
    ]


class TestDomeV3(HttpCase):
    name = "前程贷主流程测试"
    host = "http://api.lemonban.com/futureloan/"
    headers = {"X-Lemonban-Media-Type": "lemonban.v2"}
    # 用例级别前置
    setup_hook = 'setup_hook1'
    teardown_hook = 'teardown_hook1'
    # 预设变量
    env = {
        "user_mobile": 'F{rand_phone("155")}',
        "admin_mobile": 'F{rand_phone("133")}',
        "timestamp": 'F{get_timestamp()}'
    }
    # 结果校验
    verification = [
        ["eq", 200, 'status_code'],
        ["eq", {'code': 0, "msg": "OK"}, {'code': 'V{{$.code}}', 'msg': 'V{{$.msg}}'}]
    ]

    Cases = [
        # 用例1：普通用户注册
        {
            'title': "普通用户注册数据库校验",
            'interface': "member/register",
            "method": "post",
            'json': {"mobile_phone": "${{user_mobile}}", "pwd": "lemonban"},
            "db_check_hook": "register_db_check",
            "verification": [
                ["contains", 'O', "V{{$..msg}}"],
            ]
        },
        # 用例2：管理员注册
        {
            'title': "管理员注册",
            'interface': "member/register",
            "method": "post",
            'json': {"mobile_phone": "${{admin_mobile}}", "pwd": "lemonban", "type": 0}
        },
        # 用例3：普通用户登录
        {
            'title': "普通用户登录",
            'interface': "member/login",
            "method": "post",
            'json': {"mobile_phone": "${{user_mobile}}", "pwd": "lemonban"},
            "extract": {
                "token": ("env", "jsonpath", "$..token"),
                'member_id': ("env", "jsonpath", "$..id")
            }
        },
        # 用例4：充值
        {
            'title': "充值",
            'interface': "member/recharge",
            "method": "post",
            "headers": {
                "X-Lemonban-Media-Type": "lemonban.v2",
                "Authorization": "Bearer ${{token}}"
            },

            'json': {
                "member_id": '${{member_id}}',
                "amount": 20000,
                "timestamp": "${{timestamp}}",
                "sign": "F{get_sign(${{timestamp}},${{token}})}"
            }
        },
        # 用例5：管理员登录
        {
            'title': "管理员用户登录",
            'interface': "member/login",
            "method": "post",
            'json': {"mobile_phone": "${{admin_mobile}}", "pwd": "lemonban"},
            "extract": {
                "admin_token": ("env", "jsonpath", "$..token"),
                'admin_member_id': ("env", "jsonpath", "$..id")
            }
        },
        # 用例6：管理员添加项目
        {
            'title': "管理员添加项目",
            'interface': "loan/add",
            "method": "post",
            "timestamp": "F{get_timestamp()}",
            'json': {
                "member_id": '${{admin_member_id}}',
                "title": "世界这么大，借钱去看看",
                "amount": 2000.00,
                "loan_rate": 18.0,
                "loan_term": 6,
                "loan_date_type": 1,
                "bidding_days": 10,
                "timestamp": "${{timestamp}}",
                "sign": "F{get_sign(${{timestamp}},${{admin_token}})}"
            },
            "headers": {
                "X-Lemonban-Media-Type": "lemonban.v2",
                "Authorization": "Bearer ${{admin_token}}"
            },
            "extract": {
                'loan_id': ("env", "jsonpath", "$..id")
            }
        },
        # 用例7：管理员审核项目
        {
            'title': "管理员审核项目",
            'interface': "loan/audit",
            "method": "patch",
            "headers": {
                "X-Lemonban-Media-Type": "lemonban.v2",
                "Authorization": "Bearer ${{admin_token}}"
            },

            'json': {
                "loan_id": '${{loan_id}}',
                "approved_or_not": True,
                "timestamp": "${{timestamp}}",
                "sign": "F{get_sign(${{timestamp}},${{admin_token}})}"

            },
        },
        # 用例8：普通用户投资
        {
            'title': "普通用户投资",
            'interface': "member/recharge",
            "method": "post",
            "headers": {
                "X-Lemonban-Media-Type": "lemonban.v2",
                "Authorization": "Bearer ${{token}}"
            },

            'json': {
                "member_id": '${{member_id}}',
                "loan_id": '${{loan_id}}',
                "amount": 200,
                "timestamp": "${{timestamp}}",
                "sign": "F{get_sign(${{timestamp}},${{token}})}"
            }
        },
        # 用例9：取现
        {
            'title': "取现",
            'interface': "member/withdraw",
            "method": "post",
            "headers": {
                "X-Lemonban-Media-Type": "lemonban.v2",
                "Authorization": "Bearer ${{token}}"
            },
            'json': {
                "member_id": '${{member_id}}',
                "amount": 2000,
                "timestamp": "${{timestamp}}",
                "sign": "F{get_sign(${{timestamp}},${{token}})}"
            },
        }
    ]



"""
    apin中内置了一些常用生成数据和 数据加密的工具函数,在apin.core.tools中,默认全部
导入到该文件中了，如果你不需要，可以把下面from apin.core.tools import * 这行
代码注释掉。

    apin作者的建议：如果你编写的钩子函数比较多,你可以在项目创建一个hook目录，在
hooks目录中创建多个python文件来编写你的钩子函数,然后导入到本文件中,这样后期
管理钩子函数更方便。

本文件中为给大家自动各类钩子函数定义的Demo,使用时参照demo去定义，在钩子函数实现自己的需求即可。

"""
from apin.core.tools import *



# -----------------工具函数的定义Demo--------------------
# 自定义的工具函数，可以在用例文件中可以直接通过F{函数()}来调用,也可以根据需求钩子函数中调用。
def random_phone():
    """随机生产一个135开头的手机号"""
    import random
    phone = '130'
    for i in range(8):
        phone += str(random.randint(0, 9))
    return str(phone)


def md5_encrypt(data):
    """
    md5加密的工具函数
    :param data: 加密的数据
    :return:
    """
    from hashlib import md5
    new_md5 = md5()
    new_md5.update(data.encode(encoding='utf-8'))
    return new_md5.hexdigest()


# ----------------前后置钩子函数定义的demo----------------
# 前后置钩子函数在用例文件中可以通过apin预留的前后置字段去指定

def setup_hook_demo(test, ENV, env, db):
    """
    用例级别的前置猴子函数
    :param test: 用例对象
    :param ENV: 全局变量
    :param env: 局部变量
    :param db:数据库操作对象
    :return:None
    """
    pass


def teardown_hook_demo(test, ENV, env, db, response):
    """
    用例级别的后置的钩子函数
    :param test: 用例对象
    :param ENV: 全局变量
    :param env: 局部变量
    :param db:数据库操作对象
    :param response: 请求完接口的响应对象
    :return:None
    """
    pass


def setup_class_hook_demo(ENV, env, db):
    """
    测试集级别的前置的钩子函数
    :param ENV: 用来接收全局环境变量
    :param env: 用来接收全局环境变量
    :param db:数据库操作对象
    :return: None
    """
    pass


def teardown_class_hook_demo(ENV, env, db):
    """
    测试集级别的后置的钩子函数
    :param ENV: 用来接收全局环境变量
    :param env: 用来接收全局环境变量
    :param db:数据库操作对象
    :return: None
    """
    pass


# -----------------------数据库校验钩子函数定义的demo-------------------------------
# 在用例文件中可以通过db_check_hook去指定
def db_check_hook_demo(test, ENV, env, db):
    """
    数据库校验的钩子函数
    :param test: 用例对象
    :param ENV: 全局变量
    :param env: 局部变量
    :param db:数据库操作对象
    :return: 校验规则 [['eq','预期结果','实际结果']]
    yield之前写用例执行之前的sql
    yield之后写用例执行之后的sql

    Demo:
    def register_db_check(test, db, ENV, env):
        sql = "SELECT count(*) as count FROM user"
        # 前置sql查询
        s_count = db.qcd.execute(sql)['count']
        yield
        # 后置sql查询
        e_count = db.qcd.execute(sql)['count']
        # 返回sql校验表达式
        return [
            ['eq', 1, e_count-s_count]
        ]

    """
    yield



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com

import importlib
import os
import sys
from apin.core.logger import Logger
from apin.core.DBClient import DBClient

sys.path.append(os.getcwd())
sys.path.append(os.path.abspath('..'))
try:
    func_tools = importlib.import_module('funcTools')
except ModuleNotFoundError:
    from apin.core import tools as func_tools


class Settings:
    LOG_FILE_PATH = None
    DEBUG = True
    DB = []
    ENV = {}
    THREAD_COUNT = 1


try:
    settings = importlib.import_module('settings')
except:
    settings = Settings

if settings.DEBUG:
    log = Logger(path=getattr(settings, 'LOG_FILE_PATH', None),
                 level='DEBUG')
else:
    log = Logger(path=getattr(settings, 'LOG_FILE_PATH', None),
                 level='INFO')


class BaseEnv(dict):

    def __init__(self, name, **kwargs):
        self.__name = name
        super().__init__(**kwargs)

    def __setitem__(self, key, value):
        if self.__name == 'ENV':
            log.debug('设置全局变量:\n{:<10}: {}'.format(key, value))
        else:
            log.debug('设置局部变量:\n{:<10}: {}'.format(key, value))
        super().__setitem__(key, value)

    def __setattr__(self, key, value):
        super().__setattr__(key, value)
        if key != '__name' and key != '_BaseEnv__name':
            self.__setitem__(key, value)

    def __getattr__(self, item):
        if item != '__name' and item != '_BaseEnv__name':
            return super().__getitem__(item)
        return super().__getattribute__(item)

    def __delattr__(self, item):
        if item != '__name' and item != '_BaseEnv__name':
            super().__delitem__(item)
        else:
            super().__delattr__()


ENV = BaseEnv('ENV')
ENV.update(getattr(settings, 'ENV', {}))
DB = DBClient(getattr(settings, 'DB', []))



"""
============================
Author:柠檬班-木森
Time:2021/5/25 20:16
E-mail:3247119728@qq.com
=======
"""
import os
from apin.core.initEvn import ENV
from apin.core.testRunner import TestRunner
from apin.core.generateCase import ParserDataToCase


def run_test(env_config, case_data,
             func_tools_path=None,
             no_report=False,
             filename="reports.html",
             report_dir=".",
             title='测试报告',
             tester='测试员',
             desc="项目测试生成的报告",
             templates=1,
             thread_count=1,
             rerun=0,
             interval=2,
             ):
    """
    :param env_config: 全局环境变量
    :param case_data: 测试套件数据
    :param func_tools: 工具函数模块路径
    :param filename: 报告文件名
    :param report_dir:报告文件的路径
    :param title:测试套件标题
    :param templates: 可以通过参数值1或者2，指定报告的样式模板，目前只有两个模板
    :param tester:测试者
    :param thread_count:运行线程数
    :param rerun:失败重运行次数
    :param interval:重运行间隔事件
    :return:
    """
    if func_tools_path:
        with open(func_tools_path, 'rb') as f1, open('funcTools.py', 'wb') as f2:
            f2.write(f1.read())
    ENV.update(env_config)
    suite = ParserDataToCase.parser_data_create_cases([case_data])
    runner = TestRunner(suite=suite,
                        filename=filename,
                        report_dir=report_dir,
                        title=title,
                        tester=tester,
                        desc=desc,
                        templates=templates,
                        no_report=no_report
                        )
    res = runner.run(thread_count=thread_count, rerun=rerun, interval=interval)
    if func_tools_path:
        os.remove('funcTools.py')
    return res


if __name__ == '__main__':
    from apin import run_test

    case_data =[{
        "host": "http://httpbin.org",
        "headers": {
            "X-Lemonban-Media-Type": "lemonban.v2"
        },

        "env": {
            "user_mobile": "F{rand_phone()}",
            "admin_mobile": "F{rand_phone()}"
        },
        "extract": {
            "token": ("env", "jsonpath", "$..json"),
            'member_id': ("env", "jsonpath", "$..data")
        },
        "verification": [
            ["eq", 200, "status_code"]
        ],
        "Cases": [
            {
                "title": "测试用例",
                "interface": "/post",
                "method": "post",
                "json": {
                    "mobile_phone": "${{user_mobile}}",
                    "pwd": "lemonban"
                },
                "extract": {
                    "token": ("env", "jsonpath", "$..json"),
                    'member_id': ("ENV", "jsonpath", "$..headers")
                }
            },
            {
                "title": "json-demo-2",
                "interface": "/post",
                "method": "post",
                "json": {
                    "mobile_phone": "${{admin_mobile}}",
                    "pwd": "lemonban",
                    "type": 0
                },

            }
        ]
    }]
    ENV = {}
    res = run_test(env_config=ENV,
                   case_data=case_data,
                   func_tools_path=r'C:\project\MSUnitTestReport\TestApi\tests\funcTools.py',
                   no_report=True)
    print(res)



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import re
import traceback
import unittest
import time


class TestResult(unittest.TestResult):
    """ 测试结果记录"""

    def __init__(self):
        super().__init__()

        self.fields = {
            "success": 0,
            "all": 0,
            "fail": 0,
            "skip": 0,
            "error": 0,
            "begin_time": "",
            "results": [],
            "testClass": set()
        }

    def startTest(self, test):
        """
        当测试用例测试即将运行时调用
        :return:
        """
        super().startTest(test)
        self.start_time = time.time()
        getattr(test, 'info_log')("开始执行用例：【{}】\n".format(test.__doc__ or test))

    def stopTest(self, test):
        """
        当测试用列执行完成后进行调用
        :return:
        """
        # 获取用例的执行时间
        test.run_time = '{:.3}s'.format((time.time() - self.start_time))
        test.class_name = test.__class__.__qualname__
        test.method_name = test.__dict__['_testMethodName']
        test.method_doc = test.shortDescription()
        self.fields['results'].append(test)
        self.fields["testClass"].add(test.name)

    def stopTestRun(self, title=None):
        """
        测试用例执行完手动调用统计测试结果的相关数据
        :param title:
        :return:
        """
        self.fields['fail'] = len(self.failures)
        self.fields['error'] = len(self.errors)
        self.fields['skip'] = len(self.skipped)
        self.fields['all'] = sum(
            [self.fields['fail'], self.fields['error'], self.fields['skip'], self.fields['success']])
        self.fields['testClass'] = list(self.fields['testClass'])

    def addSuccess(self, test):
        """用例执行通过，成功数量+1"""
        self.fields["success"] += 1
        test.state = '成功'
        getattr(test, 'info_log')("{}执行——>【通过】\n".format(test))
        test.run_info = getattr(test, 'base_info', None)

    def addFailure(self, test, err):
        """
        :param test: 测试用例
        :param err:  错误信息
        :return:
        """
        super().addFailure(test, err)
        test.state = '失败'
        # 保存错误信息
        getattr(test, 'warning_log')(err[1])
        getattr(test, 'warning_log')("{}执行——>【失败】\n".format(test))
        test.run_info = getattr(test, 'base_info', None)

    def addSkip(self, test, reason):
        """
        修改跳过用例的状态
        :param test:测试用例
        :param reason: 相关信息
        :return: None
        """
        super().addSkip(test, reason)
        test.state = '跳过'
        getattr(test, 'info_log')("{}执行--【跳过Skip】\n".format(test))
        test.run_info = reason

    def addError(self, test, err):
        """
        修改错误用例的状态
        :param test: 测试用例
        :param err:错误信息
        :return:
        """
        super().addError(test, err)
        test.state = '错误'
        getattr(test, 'error_log')(err[1])
        getattr(test, 'error_log')("{}执行——>【错误Error】\n".format(test))
        if test.__class__.__qualname__ == '_ErrorHolder':
            test.run_time = 0
            res = re.search(r'(.*)\(.*\.(.*)\)', getattr(test, 'description'))
            test.class_name = res.group(2)
            test.method_name = res.group(1)
            test.method_doc = test.shortDescription()
            self.fields['results'].append(test)
            self.fields["testClass"].add(test.class_name)
        test.run_info = getattr(test, 'base_info', None)


class ReRunResult(TestResult):

    def __init__(self, count, interval):
        super().__init__()
        self.count = count
        self.interval = interval
        self.run_cases = []

    def startTest(self, test):
        if not hasattr(test, "count"):
            super().startTest(test)

    def stopTest(self, test):
        if test not in self.run_cases:
            self.run_cases.append(test)
            super().stopTest(test)

    def addFailure(self, test, err):
        """
        :param test: 测试用例
        :param err:  错误信息
        :return:
        """
        if not hasattr(test, 'count'):
            test.count = 0
        if test.count < self.count:
            test.count += 1
            getattr(test, 'warning_log')("{}执行——>【失败Failure】\n".format(test))
            getattr(test, 'warning_log')(err[1])
            getattr(test, 'warning_log')("开始第{}次重运行\n".format(test.count))
            time.sleep(self.interval)
            test.run(self)
        else:
            super().addFailure(test, err)
            if test.count != 0:
                getattr(test, 'warning_log')("重运行{}次完毕\n".format(test.count))

    def addError(self, test, err):
        """
        修改错误用例的状态
        :param test: 测试用例
        :param err:错误信息
        :return:
        """
        if not hasattr(test, 'count'):
            test.count = 0
        if test.count < self.count:
            test.count += 1
            getattr(test, 'error_log')("{}执行——>【错误Error】\n".format(test))
            getattr(test, 'exception_log')(err[1])
            getattr(test, 'error_log')("================{}重运行第{}次================\n".format(test, test.count))
            time.sleep(self.interval)
            test.run(self)
        else:
            super().addError(test, err)
            if test.count != 0:
                getattr(test, 'info_log')("================重运行{}次完毕================\n".format(test.count))



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import unittest
import json
import os
import yaml
from apin.core.httptest import HttpCase
from apin.core.basecase import GenerateTest


class ParserDataToCase:
    @staticmethod
    def parser_json_create_cases(dir: str):
        """解析json文件创建用例"""
        suite = unittest.TestSuite()
        if os.path.isfile(dir) and dir.endswith('.json'):
            files = [os.path.split(dir)[1]]
            dir = os.path.split(dir)[0]
        elif os.path.isdir(dir):
            files = os.listdir(dir)
        else:
            return suite
        load = unittest.TestLoader()
        for filename in files:
            if filename.endswith('.json') and filename.startswith('test'):
                with open(os.path.join(dir, filename), 'rb') as f:
                    datas = json.load(f)
                cls_name = filename.replace('.json', '').replace('test', '').split('_')
                cls_name = 'Test' + ''.join([i.capitalize() for i in cls_name])
                index = 1
                for data in datas:
                    if isinstance(data, dict):
                        cls_name = cls_name + str(index)
                        index += 1
                        cls = GenerateTest(cls_name, (HttpCase,), data)
                        suite.addTest(load.loadTestsFromTestCase(cls))
        return suite

    @staticmethod
    def parser_yaml_create_cases(dir: str):
        """解析yaml文件创建用例"""

        suite = unittest.TestSuite()
        load = unittest.TestLoader()
        if os.path.isfile(dir) and dir.endswith('.yaml'):
            files = [os.path.split(dir)[1]]
            dir = os.path.split(dir)[0]
        elif os.path.isdir(dir):
            files = os.listdir(dir)
        else:
            return suite
        for filename in files:
            # 支持yaml
            if filename.endswith('.yaml') and filename.startswith('test'):
                with open(os.path.join(dir, filename), 'rb') as f:
                    datas = yaml.load(f, Loader=yaml.FullLoader)
                cls_name = filename.replace('.yaml', 'Yaml').replace('test', '').split('_')
                cls_name = 'Test' + ''.join([i.capitalize() for i in cls_name])
                index = 1
                for data in datas:
                    if isinstance(data, dict) and data.get('testSet'):
                        cls_name = cls_name + str(index)
                        index += 1
                        cls = GenerateTest(cls_name, (HttpCase,), data.get('testSet'))
                        suite.addTest(load.loadTestsFromTestCase(cls))
        return suite

    @staticmethod
    def parser_data_create_cases(datas):
        """
        解析数据创建用例
        :param data:
        :return:
        """
        suite = unittest.TestSuite()
        load = unittest.TestLoader()
        for index, data in enumerate(datas):
            cls_name = 'TestCase{}'.format(index)
            cls = GenerateTest(cls_name, (HttpCase,), data)
            suite.addTest(load.loadTestsFromTestCase(cls))
        return suite



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import argparse
import os
import time
import unittest
from shutil import copytree
import apin
from apin.core.initEvn import settings, log, ENV
from apin.core.testRunner import TestRunner
from apin.core.logger import print_info
from apin.core.generateCase import ParserDataToCase


def create(args):
    """创建项目"""
    if hasattr(args, "name"):
        name = args.name
        template_path = os.path.join(os.path.dirname(os.path.abspath(apin.__file__)), 'templates')
        api_templates = os.path.join(template_path, 'http_demo')
        print_info("正在创建api自动化项目:{}".format(name))
        try:
            copytree(api_templates, os.path.join(".", name))
        except Exception as e:
            print_info("项目创建失败！:{}".format(e))
        else:
            print_info("项目创建成功！")


def run(args=None):
    """run test"""
    log.info('开始执行测试，加载用例....')
    if isinstance(args, argparse.Namespace):
        args = args
    else:
        if args is None:
            args = create_parser().parse_args(['run'])
        elif args[0] != 'run':
            args = create_parser().parse_args(['run'] + args)
        else:
            args = create_parser().parse_args(args)
    if hasattr(args, "file"):
        dir = os.path.abspath(getattr(args, 'file'))
    else:
        # help info
        dir = os.path.abspath('.')
    if os.path.isfile(dir):
        if dir.endswith('.yaml'):
            suite = ParserDataToCase.parser_yaml_create_cases(dir)
        elif dir.endswith('.json'):
            suite = ParserDataToCase.parser_json_create_cases(dir)
        elif dir.endswith('.py'):
            suite = unittest.defaultTestLoader.discover(os.path.dirname(dir), pattern=os.path.split(dir)[1])
        else:
            suite = unittest.TestSuite()
    elif os.path.isdir(dir):
        # load TestCase
        # load yaml TestCase
        yaml_dir = os.path.join(dir, 'case_yaml')
        suite1 = ParserDataToCase.parser_yaml_create_cases(yaml_dir)
        # load json TestCase
        json_dir = os.path.join(dir, 'case_json')
        suite2 = ParserDataToCase.parser_json_create_cases(json_dir)
        # load py TestCase
        case_dir = os.path.join(dir, 'case_py')
        suite3 = unittest.defaultTestLoader.discover(case_dir)
        suite = unittest.TestSuite()
        suite.addTest(suite1)
        suite.addTest(suite2)
        suite.addTest(suite3)
    else:
        suite = unittest.TestSuite()
    if suite == unittest.TestSuite():
        log.error('未加载到用例,请确认指定的用例路径或者目录是否正确！')
        log.error('执行路径为：{}'.format(dir))
        return
    # Test Result
    result = getattr(settings, 'TEST_RESULT', {})
    report_name = result.get('filename') if result.get('filename') else "report.html"
    result['filename'] = time.strftime('%Y-%m-%d_%H_%M_%S') + report_name

    runner = TestRunner(suite=suite, **result)
    res = runner.run(thread_count=getattr(args, "thread") or getattr(settings, 'THREAD', 1),
                     rerun=getattr(args, "rerun") or getattr(settings, 'RERUN', 0),
                     interval=getattr(args, "interval") or getattr(settings, 'INTERVAL', 2)
                     )

    if hasattr(settings, 'EMAIL'):
        try:
            # 邮件通知处理配置
            for k, v in getattr(settings, 'EMAIL').items():
                if not v:
                    raise ValueError("邮件参数配置有误,setting.py文件中的EMAIL的 {} 字段不能为空".format(k))
            else:
                runner.send_email(**getattr(settings, 'EMAIL'))
        except Exception  as e:
            log.error("测试结果邮件推送失败：{}".format(e))
        else:
            log.info("测试结果已发送到邮箱")
    if hasattr(settings, 'DINGTALK'):
        try:
            res = runner.dingtalk_notice(**settings.DINGTALK)
        except Exception as e:
            log.error("发送钉钉通知出错了，错误信息如下:{}".format(e))

        else:
            if res["errcode"] == 0:
                log.info("测试结果推送到钉钉群成功！")
            else:
                log.error("测试结果推送到钉钉群失败！错误信息:  {}".format(res["errmsg"]))
    if hasattr(settings, 'WECHAT'):
        try:
            res = runner.weixin_notice(**settings.WECHAT)
        except Exception as e:
            log.error("测试结果推送到企业微信出错了，错误信息如下:  {}".format(e))
        else:
            if res["errcode"] == 0:
                log.info("测试结果推送到企业微信成功！")
            else:
                log.error("测试结果推送到企业微信失败！错误信息： {}".format(res["errmsg"]))

    return res


def create_parser():
    parser = argparse.ArgumentParser(prog='apin', description='ApiTest使用命令介绍')
    # 添加版本号
    parser.add_argument('-V', '--version', action='version', version='%(prog)s 1.1.3')
    subparsers = parser.add_subparsers(title='Command', metavar="命令")
    # 创建项目命令
    create_cmd = subparsers.add_parser('create', help='create test project ', aliases=['C'])
    create_cmd.add_argument('name', metavar='project_name', help="project name")
    create_cmd.set_defaults(func=create)
    # 运行项目命令
    parser_run = subparsers.add_parser('run', help='run test project', aliases=['R'])
    parser_run.add_argument('--file', type=str, metavar='file', help='case file path', default='.')
    parser_run.add_argument('--thread', type=int, metavar='thread', help='Concurrent running thread', default=None)
    parser_run.add_argument('--rerun', type=int, metavar='rerun', help='Number of failed case reruns', default=None)
    parser_run.add_argument('--interval', type=int, metavar='interval', help='Rerun interval', default=None)
    parser_run.set_defaults(func=run)
    return parser


def main(params: list = None):
    """
    程序入口
    :param params: list
    :return:
    """
    parser = create_parser()
    # 获取参数
    if params:
        args = parser.parse_args(params)
    else:
        args = parser.parse_args()
    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()


def run_test(env_config, case_data,
             func_tools_path=None,
             no_report=False,
             filename="reports.html",
             report_dir=".",
             title='测试报告',
             tester='测试员',
             desc="项目测试生成的报告",
             templates=1,
             thread_count=1,
             rerun=0,
             interval=2,
             ):
    """
    :param env_config: 全局环境变量
    :param case_data: 测试套件数据
    :param func_tools: 工具函数模块路径
    :param filename: 报告文件名
    :param report_dir:报告文件的路径
    :param title:测试套件标题
    :param templates: 可以通过参数值1或者2，指定报告的样式模板，目前只有两个模板
    :param tester:测试者
    :return:
    """
    if func_tools_path:
        with open(func_tools_path, 'rb') as f1, open('funcTools.py', 'wb') as f2:
            f2.write(f1.read())
    ENV.update(env_config)
    suite = ParserDataToCase.parser_data_create_cases(case_data)
    runner = TestRunner(suite=suite,
                        filename=filename,
                        report_dir=report_dir,
                        title=title,
                        tester=tester,
                        desc=desc,
                        templates=templates,
                        no_report=no_report
                        )
    res = runner.run(thread_count=thread_count, rerun=rerun, interval=interval)
    if func_tools_path:
        os.remove('funcTools.py')
    return res


if __name__ == '__main__':
    main(['create', '../demo2'])



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import hmac
import hashlib
import base64
import urllib.parse
import requests
import os
import smtplib
import time
from email.mime.text import MIMEText
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart


from requests import utils

class SendEmail:
    """发送邮件"""

    def __init__(self, host, user, password, port=465):
        """
        初始化设置
        :param host: smtp服务器地址（qq邮箱：smtp.qq.com，163邮箱：smtp.163.com"）
        :param port: smtp服务器端口：
        :param user: 邮箱账号
        :param password: 邮箱的smtp服务授权码
        """
        if port == 465 or port == 587:
            self.smtp = smtplib.SMTP_SSL(host=host, port=port)
        else:
            self.smtp = smtplib.SMTP(host=host, port=port)
        self.smtp.login(user=user, password=password)
        self.user = user

    def send_email(self, subject="测试报告", content=None, filename=None, to_addrs=None):
        """
        发送邮件
        :param subject: 邮件主题
        :param content: 邮件内容
        :param filename: 报告文件的完整路径
        :param to_addrs: 收件人地址
        :type to_addrs: str or list
        :return:
        """
        print("--------准备发送测试报告---------")
        msg = MIMEMultipart()
        msg["Subject"] = subject
        msg["From"] = self.user
        if isinstance(to_addrs, str):
            msg["To"] = to_addrs
        elif to_addrs and isinstance(to_addrs, list):
            msg["To"] = to_addrs[0]
        if not content:
            content = time.strftime("%Y-%m-%d-%H_%M_%S") + ":测试报告"
        # 构建邮件的文本内容
        text = MIMEText(content, _subtype="html", _charset="utf8")
        msg.attach(text)
        # 判断是否要发送附件
        if filename and os.path.isfile(filename):
            with open(filename, "rb") as f:
                content = f.read()
            try:
                report = MIMEApplication(content, _subtype=None)
            except:
                report = MIMEApplication(content)
            name = os.path.split(filename)[1]
            report.add_header('content-disposition', 'attachment', filename=name)
            msg.attach(report)
        # 第三步：发送邮件
        try:
            self.smtp.send_message(msg, from_addr=self.user, to_addrs=to_addrs)
        except Exception as e:
            print("--------测试报告发送失败------")
            raise e
        else:
            print("--------测试报告发送完毕------")


class DingTalk:
    """发生钉钉群通知"""

    def __init__(self, url, data, secret=None):
        """
        初始化机器人对象
        :param url:钉钉机器人webhook地址
        :param data:发送的消息（参照钉钉官方的消息类型）
        :param secret: (非必填)如果机器人安全设置了加签，则需要传入加签的秘钥
        """
        self.url = url
        self.data = data
        self.secret = secret

    def get_stamp(self):
        """加签"""
        timestamp = str(round(time.time() * 1000))
        secret_enc = self.secret.encode('utf-8')
        string_to_sign = '{}\n{}'.format(timestamp, self.secret)
        string_to_sign_enc = string_to_sign.encode('utf-8')
        hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
        return {"sign": sign, "timestamp": timestamp}

    def send_info(self):
        """发送消息"""
        # 判断是否需要加签
        if self.secret:
            params = self.get_stamp()
        else:
            params = None
        # 发送请求
        response = requests.post(url=self.url, json=self.data, params=params)
        return response


class WeiXin:
    """
    企业微信群通知
    """
    base_url = "https://qyapi.weixin.qq.com/cgi-bin/appchat/send?access_token="

    def __init__(self, access_token=None, corpid=None, corpsecret=None):
        """
        :param corpid:企业ID，
        :param corpsecret:应用的凭证密钥
        """
        self.corpid = corpid
        self.corpsecret = corpsecret
        if access_token:
            self.access_token = access_token
        elif corpid and corpsecret:
            self.access_token = self.get_access_token()
        else:
            raise ValueError("access_token和【corpid,corpsecret】不能都为空，至少要传入一种")

    def get_access_token(self):
        """获取access_token"""
        url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken"
        params = {
            "corpid": self.corpid,
            "corpsecret": self.corpsecret
        }
        result = requests.get(url=url, params=params).json()
        if result.json()['errcode'] != 0:
            raise ValueError(result["errmsg"])
        return result["access_token"]

    def send_info(self, data):
        """发送消息"""
        url = self.base_url + self.access_token
        response = requests.post(url=url, data=data)
        return response


if __name__ == '__main__':
    url = "https://oapi.dingtalk.com/robot/send?access_token=690900b5ce6d5d10bb1218b8e64a4e2b55f96a6d116aaf50"
    data = {
        "msgtype": "markdown",
        "markdown": {
            "title": "自动化测试报告",
            "text": open('python31.md', 'r', encoding='utf-8').read()
        },
        "at": {
            "atMobiles": [],
            "isAtAll": False
        }
    }
    ding = DingTalk(url=url, data=data)
    res = ding.send_info()
    print(res)



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import json
import os
import unittest
import time
import copy
from jinja2 import Environment, FileSystemLoader
from concurrent.futures.thread import ThreadPoolExecutor
from apin.core.initEvn import log
from apin.core.testResult import ReRunResult
from apin.core.resultPush import DingTalk, WeiXin, SendEmail



class TestRunner():
    """unittest运行程序"""

    def __init__(self, suite: unittest.TestSuite,
                 filename="reports.html",
                 report_dir=".",
                 title='测试报告',
                 tester='测试员',
                 desc="XX项目测试生成的报告",
                 templates=1,
                 no_report=False
                 ):
        """
        初始化用例运行程序
        :param suites: 测试套件
        :param filename: 报告文件名
        :param report_dir:报告文件的路径
        :param title:测试套件标题
        :param templates: 可以通过参数值1或者2，指定报告的样式模板，目前只有两个模板
        :param tester:测试者
        :param no_report:不生成测试报告，以json数据格式返回测试结果，默认生成，设置True则不生成报告
        """
        if not isinstance(suite, unittest.TestSuite):
            raise TypeError("suites 不是测试套件")
        if not isinstance(filename, str):
            raise TypeError("filename is not str")
        if not filename.endswith(".html"):
            filename = filename + ".html"
        self.suite = suite
        self.filename = filename
        self.title = title
        self.tester = tester
        self.desc = desc
        self.templates = templates
        self.report_dir = report_dir
        self.result = []
        self.starttime = time.time()
        self.no_report = no_report

    def __classification_suite(self):
        """
        将测试套件中的用例，根据用例类位单位，拆分成多个测试套件，打包成列表类型
        :return: list-->[suite,suite,suite.....]
        """
        suites_list = []

        def wrapper(suite):
            for item in suite:
                if isinstance(item, unittest.TestCase):
                    suites_list.append(suite)
                    break
                else:
                    wrapper(item)

        wrapper(copy.deepcopy(self.suite))
        return suites_list

    def __get_reports(self,thread_count):
        """
        生成报告,返回测试汇中的结果
        :return: 包含测试结果的字典
        """
        # 汇总测试结果
        test_result = {
            "success": 0,
            "all": 0,
            "fail": 0,
            "skip": 0,
            "error": 0,
            "results": [],
            "testClass": [],

        }

        # 整合测试结果
        for res in self.result:
            for item in test_result:
                test_result[item] += res.fields[item]
        test_result['thread_count'] =thread_count
        test_result['runtime'] = '{:.2f} S'.format(time.time() - self.starttime)
        test_result["begin_time"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.starttime))
        test_result["title"] = self.title
        test_result["tester"] = self.tester
        test_result['desc'] = self.desc
        if test_result['all'] != 0:
            test_result['pass_rate'] = '{:.2f}'.format(test_result['success'] / test_result['all'] * 100)
        else:
            test_result['pass_rate'] = 0
        log.info("用例运行完毕,结果如下：\n共运行：{}条 "
                 "\n通过:{}条"
                 "\n失败:{}条"
                 "\n错误:{}条"
                 "\n运行时间:{}".format(
            test_result['all'], test_result['success'], test_result['fail'], test_result['error'],
            test_result['runtime']
        ))
        self.test_result = test_result
        # 判断是否要生产测试报告
        if os.path.isdir(self.report_dir):
            pass
        else:
            os.mkdir(self.report_dir)
        if self.no_report:
            return self.__get_results(test_result)
        log.info("正在生成测试报告中......")

        # 获取历史执行数据
        test_result['history'] = self.__handle_history_data(test_result)

        # 获取报告模板
        template_path = os.path.join(os.path.dirname(__file__), '../templates/reports')
        env = Environment(loader=FileSystemLoader(template_path))
        if self.templates == 2:
            template = env.get_template('templates2.html')
        elif self.templates == 3:
            template = env.get_template('templates3.html')
        else:
            template = env.get_template('templates1.html')

        file_path = os.path.join(self.report_dir, self.filename)
        # 渲染报告模板
        res = template.render(test_result)
        # 输出报告到文件
        with open(file_path, 'wb') as f:
            f.write(res.encode('utf8'))
        log.info("测试报告已经生成，报告路径为:{}".format(file_path))
        self.email_conent = {"file": os.path.abspath(file_path),
                             "content": env.get_template('templates03.html').render(test_result)
                             }

        return {'success': test_result['success'],
                'all': test_result['all'],
                'fail': test_result['fail'],
                'skip': test_result['skip'],
                'error': test_result['error'],
                'runtime': test_result['runtime'],
                'begin_time': test_result['begin_time'],
                'tester': test_result['tester'],
                'pass_rate': test_result['pass_rate'],
                'report': file_path,
                "thread_count":thread_count
                }

    def __handle_history_data(self, test_result):
        """
        处理历史数据
        :return:
        """
        try:
            with open(os.path.join(self.report_dir, 'history.json'), 'r', encoding='utf-8') as f:
                history = json.load(f)
        except FileNotFoundError as e:
            history = []
        history.append({'success': test_result['success'],
                        'all': test_result['all'],
                        'fail': test_result['fail'],
                        'skip': test_result['skip'],
                        'error': test_result['error'],
                        'runtime': test_result['runtime'],
                        'begin_time': test_result['begin_time'],
                        'pass_rate': test_result['pass_rate'],
                        })

        with open(os.path.join(self.report_dir, 'history.json'), 'w', encoding='utf-8') as f:
            json.dump(history, f, ensure_ascii=True)
        return history

    def __get_notice_content(self):
        """获取通知的内容"""
        template_path = os.path.join(os.path.dirname(__file__), '../templates/reports')
        env = Environment(loader=FileSystemLoader(template_path))
        res_text = env.get_template('dingtalk.md').render(self.test_result)
        return res_text

    def __get_results(self, test_result):
        """返回测试结果"""
        results = []
        for case in test_result.get('results'):
            results.append({k: v for k, v in case.__dict__.items() if not k.startswith('_')})
        test_result['results'] = results
        return test_result

    def run(self, thread_count=1, rerun=0, interval=2):
        """
        支持多线程执行
        注意点：如果多个测试类共用某一个全局变量，由于资源竞争可能会出现错误
        :param thread_count:线程数量，默认位1
        :return:测试运行结果
        """
        # 将测试套件按照用例类进行拆分
        suites = self.__classification_suite()
        with ThreadPoolExecutor(max_workers=thread_count) as ts:
            for i in suites:
                res = ReRunResult(count=rerun, interval=interval)
                self.result.append(res)
                ts.submit(i.run, result=res).add_done_callback(res.stopTestRun)
            ts.shutdown(wait=True)
        result = self.__get_reports(thread_count)
        return result

    def send_email(self, host, port, user, password, to_addrs, is_file=True):
        """
        发生报告为附件到邮箱
        :param host: str类型，(smtp服务器地址)
        :param port: int类型，(smtp服务器地址端口)
        :param user: str类型，(邮箱账号)
        :param password: str类型（邮箱密码）
        :param to_addrs: str(单个收件人) or list(多个收件人)收件人列表，
        :return:
        """
        sm = SendEmail(host=host, port=port, user=user, password=password)
        if is_file:
            filename = self.email_conent["file"]
        else:
            filename = None
        content = self.email_conent["content"]

        sm.send_email(subject=self.title, content=content, filename=filename, to_addrs=to_addrs)

    def get_except_info(self):
        """
        获取错误用例和失败用例的报错信息
        :return:
        """
        except_info = []
        num = 0
        for i in self.result:
            for texts in i.failures:
                t, content = texts
                num += 1
                except_info.append("*{}、用例【{}】执行失败*，\n失败信息如下：".format(num, t._testMethodDoc))
                except_info.append(content)
            for texts in i.errors:
                num += 1
                t, content = texts
                except_info.append("*{}、用例【{}】执行错误*，\n错误信息如下：".format(num, t._testMethodDoc))
                except_info.append(content)
        except_str = "\n".join(except_info)
        return except_str

    def dingtalk_notice(self, url, key=None, secret=None, atMobiles=None, isatall=False, except_info=False):
        """
        钉钉通知
        :param url: 钉钉机器人的Webhook地址
        :param key: （非必传：str类型）如果钉钉机器人安全设置了关键字，则需要传入对应的关键字
        :param secret:（非必传:str类型）如果钉钉机器人安全设置了签名，则需要传入对应的密钥
        :param atMobiles: （非必传，list类型）发送通知钉钉中要@人的手机号列表，如：[137xxx,188xxx]
        :param isatall: 是否@所有人，默认为False,设为True则会@所有人
        :param except_info:是否发送未通过用例的详细信息，默认为False，设为True则会发送失败用例的详细信息
        :return:  发送成功返回 {"errcode":0,"errmsg":"ok"}  发送失败返回 {"errcode":错误码,"errmsg":"失败原因"}
        """
        if not url:
            raise ValueError("url(钉钉群机器人的Webhook地址)不能为空")

        res_text = self.__get_notice_content()
        if except_info:
            res_text += '\n ### 未通过用例详情：\n'
            res_text += self.get_except_info()
        data = {
            "msgtype": "markdown",
            "markdown": {
                "title": '{}({})'.format(self.title, key),
                "text": res_text
            },
            "at": {
                "atMobiles": atMobiles,
                "isAtAll": isatall
            }
        }
        ding = DingTalk(url=url, data=data, secret=secret)
        response = ding.send_info()
        return response.json()

    def weixin_notice(self, chatid, access_token=None, corpid=None, corpsecret=None):
        """
        测试结果推送到企业微信群，【access_token】和【corpid，corpsecret】至少要传一种
        可以传入access_token ,也可以传入（corpid，corpsecret）来代替access_token
        :param chatid: 企业微信群ID
        :param access_token: 调用企业微信API接口的凭证
        :param corpid: 企业ID
        :param corpsecret:应用的凭证密钥
        :return:
        """
        # 获取通知结果
        res_text = self.__get_notice_content()
        data = {
            "chatid": chatid,
            "msgtype": "markdown",
            "markdown": {
                "content": res_text
            }
        }
        wx = WeiXin(access_token=access_token, corpid=corpid, corpsecret=corpsecret)
        response = wx.send_info(data=data)
        return response



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import datetime
import os
import yaml
import json
import unittest
from functools import wraps
from apin.core.dataParser import DataParser
from apin.core.initEvn import BaseEnv, log, settings


class CaseLog:
    log = log

    def save_log(self, message, level):
        if not hasattr(self, 'log_data'):
            setattr(self, 'log_data', [])
        info = "【{}】| {} |: {}".format(level, datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'), message)
        getattr(self, 'log_data').append((level, info))

    def save_error(self, message):
        if not hasattr(self, 'error_info'):
            setattr(self, 'error_info', [])
        getattr(self, 'error_info').append("【ERROR】:{} ".format(message))

    def debug_log(self, message):
        if settings.DEBUG:
            self.save_log(message, 'DEBUG')
            self.log.debug(message)

    def info_log(self, message):
        self.save_log(message, 'INFO')
        self.log.info(message)

    def warning_log(self, message):
        self.save_log(message, 'WARNING')
        self.log.warning(message)

    def error_log(self, message):
        self.save_log(message, 'ERROR')
        self.save_error(message)
        self.log.error(message)

    def exception_log(self, message):
        self.save_log(message, 'ERROR')
        self.save_error(message)
        self.log.exception(message)

    def critical_log(self, message):
        self.save_log(message, 'CRITICAL')
        self.save_error(message)
        self.log.critical(message)


class GenerateTest(type):
    """生成用例的类"""

    def __new__(cls, name, bases, namespace, *args, **kwargs):

        if name in ('BaseTestCase', 'HttpCase'):
            return super().__new__(cls, name, bases, namespace)
        else:
            # -------------------生成用例---------------------
            # Case外的类属性中，是否有需要动态执行的函数
            for k, v in list(namespace.items()):
                if k not in ['Cases', "extract", "verification"]:
                    # 解析数据中的变量
                    v = DataParser.parser_func(namespace.get('env'), v)
                    v = DataParser.parser_variable(namespace.get('env'), v)
                    namespace[k] = v
                if k == 'env':
                    _v = BaseEnv('env')
                    _v.update(v)
                    namespace[k] = _v
            test_cls = super().__new__(cls, name, bases, namespace)
            func = getattr(test_cls, "perform")
            datas = cls.__handle_datas(namespace.get('Cases'))
            for index, case_data in enumerate(datas):
                # 生成用例名称，
                new_test_name = cls.__create_test_name(index, case_data, test_cls)
                # 生成用例描述
                if isinstance(case_data, dict) and case_data.get("title"):
                    test_desc = case_data.get("title")
                elif isinstance(case_data, dict) and case_data.get("desc"):
                    test_desc = case_data.get("desc")
                elif hasattr(case_data, 'title'):
                    test_desc = case_data.title
                else:
                    test_desc = func.__doc__
                func2 = cls.__update_func(new_test_name, case_data, test_desc, func)
                setattr(test_cls, new_test_name, func2)
            return test_cls

    @classmethod
    def __create_test_name(cls, index, case_data, test_cls):
        interface = case_data.get('interface') or getattr(test_cls, 'interface')
        if index + 1 < 10:
            test_name = 'test' + "_0" + str(index + 1) + '_' + interface.strip('/').replace('/', '_')
        else:
            test_name = 'test' + "_" + str(index + 1) + '_' + interface.strip('/').replace('/', '_')
        return test_name

    @classmethod
    def __update_func(cls, new_func_name, params, test_desc, func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            return func(self, params, *args, **kwargs)

        wrapper.__wrapped__ = func
        wrapper.__name__ = new_func_name
        wrapper.__doc__ = test_desc
        return wrapper

    @classmethod
    def __handle_datas(cls, datas):
        if isinstance(datas, list):
            return datas
        if isinstance(datas, str) and datas.endswith('.json') or datas.endswith('.yaml'):
            if os.getcwd().split() == "case_py":
                filepath = os.path.join('..', 'case_yaml', datas)
            else:
                filepath = os.path.join('.', 'case_yaml', datas)
            # 支持json
            if filepath.endswith('.json'):
                with open(filepath, 'rb') as f:
                    return json.load(f)
            # 支持yaml
            if filepath.endswith('.yaml'):
                with open(filepath, 'rb') as f:
                    return yaml.load(f, Loader=yaml.FullLoader)
            # 支持excel：暂未实现
        raise ValueError('测试用例数据格式有误！')


class BaseTestCase(unittest.TestCase, CaseLog, metaclass=GenerateTest):
    name = ''

    def perform(self, item):
        pass

    def get(self, attr):
        """支持通过get方法获取属性"""
        return getattr(self, attr, None)



"""
测试项目的配置

"""
import os

BASEDIR = os.path.dirname(os.path.abspath(__file__))
# 日志文件路径
LOG_FILE_PATH = os.path.join(BASEDIR, "./logs")

# 是否为debug模式
DEBUG = False

# 数据库配置
DB = [
    {"name": "aliyun",
     "type": "mysql",
     "config": {
         "user": "future",
         "password": "123456",
         "host": "api.lemonban.com",
         "port": 3306}
     },
    {"name": "qcd",
     "type": "mysql",
     "config": {
         'user': "future",
         "password": "123456",
         "host": "api.lemonban.com",
         "port": 3306}
     }
]

# 测试环境配置
ENV = {
    "host": "http://api.lemonban.com/futureloan/3",

}
THREAD_COUNT = 1
# 失败重运行次数
RERUN = 0
# 重运行间隔时间(秒)
INTERVAL = 2
# 测试结果配置
TEST_RESULT = {
    "report_dir": './reports',
    # 测试报告名称
    "filename": "report.html",
    # 测试人员
    "tester": "测试员",
    # 报告标题
    "title": "测试报告",
    # 报告样式
    "templates": 1,
    # 报告描述信息
    "desc": "XX项目测试生成的报告",
}

# # # 邮箱通知配置
# EMAIL = {
#     # smtp服务器地址
#     "host": '',
#     # smtp服务器端口
#     "port": "",
#     # 邮箱账号
#     "user": "",
#     # smtps授权码
#     "password": "",
#     # 收件人列表
#     "to_addrs": [],
#     # 是否发送附件
#     "is_file": True
# }
#
# # 钉钉通知
# DINGTALK = {
#     #  钉钉机器人的Webhook地址
#     "url": "",
#     # 如果钉钉机器人安全设置了关键字，则需要传入对应的关键字
#     "key": None,
#     # 如果钉钉机器人安全设置了签名，则需要传入对应的密钥
#     "secret": None,
#     # 钉钉群中要@人的手机号列表，如：[137xxx,188xxx]
#     "atMobiles": [],
#     # 是否@所有人
#     "isatall": False
# }
#
# # 企业微信通知
# WECHAT = {
#     # chatid:企业微信群ID
#     "chatid": "",
#     # access_token:调用企业微信API接口的凭证
#     "access_token": ""
# }



"""
============================
Author:柠檬班-木森
Time:2021/3/1 20:20
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
=======
"""
from apin.core.httptest import HttpCase


class TestDome1(HttpCase):
    # host地址,如果setting中定义了此处可以不写
    host = "http://httpbin.org"
    # 请求头
    headers = {
        "UserAgent": "python/apin"
    }
    # 请求方法
    method = 'get'
    # 接口地址
    interface = '/get'
    # 指定用例级别的前置钩子
    setup_hook = "setup_hook_demo"
    # 指定用例级别的猴子猴子
    teardown_hook = "teardown_hook_demo"
    # 用例断言
    verification = [
        ["eq", 200, "status_code"]
    ]
    # 用例数据
    Cases = [
        # 用例1
        {
            "title": "py文件-用例1",
            "json": {
                "mobile_phone": "17788907899",
                "pwd": "lemonban"
            },
        },
        # 用例2
        {
            "title": "py文件-用例2",
            "json": {
                "mobile_phone": "17788907899",
                "pwd": "lemonban"
            }
        }
    ]



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import json
import re
from numbers import Number
from apin.core import tools
from apin.core.initEvn import func_tools, ENV


class DataParser:
    """数据解析"""
    # 函数的规则
    func_pattern = r'F{(.*?)\((.*?)\)}'
    # 变量的规则
    variable_pattern = r'\${{.+?}}'
    var2_pattern = r'\${{(.+?)}}'

    @classmethod
    def parser_func(cls, dic_attrs, data):
        """解析函数"""
        old_data = data
        if isinstance(data, str):
            while re.search(cls.func_pattern, data):
                res2 = re.search(cls.func_pattern, data)
                item = res2.group()
                # 获取函数名
                f_name = res2.group(1)
                # 提取函数的参数
                params = cls.get_func_params(res2, dic_attrs)
                # 执行函数
                func = getattr(func_tools, f_name, None) or getattr(tools, f_name, None)
                if not func:
                    raise ValueError('函数引用错误：\n{}\n中的函数{}未定义！,'.format(data, f_name))
                # 执行函数
                value = func(*params)
                if item == data: return value
                data = data.replace(item, str(value))
            return data
        elif isinstance(data, list) or isinstance(data, dict):
            data = str(data)
            while re.search(cls.func_pattern, data):
                res2 = re.search(cls.func_pattern, data)
                item = res2.group()
                # 获取函数名
                f_name = res2.group(1)
                # 提取函数的参数
                params = cls.get_func_params(res2, dic_attrs)
                func = getattr(func_tools, f_name, None) or getattr(tools, f_name, None)
                if not func:
                    raise ValueError('函数引用错误：\n{}\n中的函数{}没有定义！,'.format(
                        json.dumps(old_data, ensure_ascii=False, indent=2), f_name)
                    )
                # 执行函数
                value = func(*params)
                if isinstance(value, Number):
                    s = data.find(item)
                    dd = data[s - 1:s + len(item) + 1]
                    data = data.replace(dd, str(value))
                elif isinstance(value, str) and "'" in value:
                    data = data.replace(item, value.replace("'", '"'))
                else:
                    data = data.replace(item, value)
            return eval(data)
        else:
            return data

    @classmethod
    def get_func_params(cls, match_obj, dic_attrs):
        """解析函数的参数"""
        params = []
        # 提取函数的参数
        if match_obj.group(2):
            for i in match_obj.group(2).strip().split(','):
                # 判断参数中是否有变量
                if re.search(cls.var2_pattern, i):
                    i = cls.parser_variable(dic_attrs, i)
                else:
                    # 判断是否双引号开头，是的话去除字符串
                    i = i[1:-1] if i.startswith('"') else eval(i)
                params.append(i)
        return params

    @classmethod
    def parser_variable(cls, dic_attrs, data):
        old_data = data
        """解析变量"""
        if isinstance(data, str):
            while re.search(cls.var2_pattern, data):
                res2 = re.search(cls.var2_pattern, data)
                item = res2.group()
                attr = res2.group(1)
                value = ENV.get(attr) if dic_attrs.get(attr) is None else dic_attrs.get(attr)
                if value is None:
                    raise ValueError('变量引用错误:\n{}中的变量{},在当前运行环境中未找到'.format(data, attr))

                if item == data: return value
                data = data.replace(item, str(value))
            return data
        elif isinstance(data, list) or isinstance(data, dict):
            data = str(data)
            while re.search(cls.var2_pattern, data):
                res2 = re.search(cls.var2_pattern, data)
                item = res2.group()
                attr = res2.group(1)
                value = ENV.get(attr) if dic_attrs.get(attr) is None else dic_attrs.get(attr)
                if value is None:
                    raise ValueError('变量引用错误：\n{}\n中的变量{},在当前运行环境中未找到'.format(
                        json.dumps(old_data, ensure_ascii=False, indent=2), attr)
                    )
                if isinstance(value, Number):
                    s = data.find(item)
                    dd = data[s - 1:s + len(item) + 1]
                    data = data.replace(dd, str(value))
                elif isinstance(value, str) and "'" in value:
                    data = data.replace(item, value.replace("'", '"'))
                else:
                    data = data.replace(item, str(value))
            return eval(data)
        else:
            return data



"""
============================
Project: Apin
Author:柠檬班-木森
Time:2021/7/9 16:44
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
Site: http://www.lemonban.com
Forum: http://testingpai.com 
============================
"""
import random
import time


def setup_hook1(test, ENV, env, db):
    """
    随机生成一个手机号码，保存为全局变量user_phone
    :param ENV: 全局变量
    :param env: 局部变量
    :return:
    """
    phone = ''
    for i in range(8):
        phone += str(random.randint(0, 9))
    env.user_phone = phone


def get_timestamp_hook(test, ENV, env, db):
    """获取时间戳,保存为局部变量"""
    env.timestamp = time.time()


def teardown_hook1(test, ENV, env, db, response):
    print('测试用例后置钩子')
    pass



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import os
import logging
from logging.handlers import TimedRotatingFileHandler, BaseRotatingHandler
import colorama

colorama.init()


class Logger:
    __instance = None
    sh = logging.StreamHandler()

    def __new__(cls, path=None, level='DEBUG', RotatingFileHandler: BaseRotatingHandler = None):
        """
        :param path: report path
        :param args:
        :param kwargs:
        :return:
        """
        if not cls.__instance:
            cls.__instance = super().__new__(cls)
            log = logging.getLogger('apin')
            log.setLevel(level)
            cls.__instance.log = log
        if path:
            if not os.path.isdir(path):
                os.mkdir(path)
            if RotatingFileHandler and isinstance(RotatingFileHandler, BaseRotatingHandler):
                fh = RotatingFileHandler
            else:
                fh = TimedRotatingFileHandler(os.path.join(path, 'logging.log'), when='d',
                                              interval=1, backupCount=7,
                                              encoding="utf-8")
            fh.setLevel(level)
            cls.__instance.log.addHandler(fh)
            # 定义handler的输出格式
            formatter = logging.Formatter("%(asctime)s | 【%(levelname)s】 | : %(message)s")
            fh.setFormatter(formatter)
        return cls.__instance

    def set_level(self, level):
        """设置日志输出的等级"""
        self.log.setLevel(level)

    def set_file_handle(self, level, path):
        if path:
            if not os.path.isdir(path):
                os.mkdir(path)
            fh = TimedRotatingFileHandler(os.path.join(path, 'apin-ui.log'), when='d',
                                          interval=1, backupCount=7,
                                          encoding="utf-8")
            fh.setLevel(level)
            self.log.addHandler(fh)
            # 定义handler的输出格式
            formatter = logging.Formatter("%(asctime)s | 【%(levelname)s】 | : %(message)s")
            fh.setFormatter(formatter)

    def debug(self, message):
        self.fontColor('\033[0;34m{}\033[0;34m{}\033[0;34m{}')
        self.log.debug(message)

    def info(self, message):
        self.fontColor('\033[0;32m{}\033[0;32m{}\033[0;32m{}')
        self.log.info(message)

    def warning(self, message):
        self.fontColor('\033[0;33m{}\033[0;43m{}\033[0;33m{}')
        self.log.warning(message)

    def error(self, message):
        self.fontColor('\033[0;31m{}\033[0;41m{}\033[0;31m{}')
        self.log.error(message)

    def exception(self, message):
        self.fontColor('\033[0;31m{}\033[0;41m{}\033[0;31m{}')
        self.log.exception(message)

    def critical(self, message):
        self.fontColor('\033[0;35m{}\033[0;45m{}\033[0;35m{}')
        self.log.critical(message)

    def fontColor(self, color):
        # 不同的日志输出不同的颜色
        formatter = logging.Formatter(color.format("%(asctime)s| ", "【%(levelname)s】", " | : %(message)s"))
        self.sh.setFormatter(formatter)
        self.log.addHandler(self.sh)


def print_info(msg):
    print('\033[0;32m{}'.format(msg))


def print_waring(msg):
    print('\033[0;33m{}'.format(msg))


def print_error(msg):
    print('\033[0;31m{}'.format(msg))


if __name__ == '__main__':
    print_info('12323')
    print_waring('12323')
    print_error('12323')

    logger = Logger()
    logger.debug('debu等级日志')
    logger.info('info日志')
    logger.warning('warning日志')
    logger.error('error日志')
    logger.critical('CRITICAL日志')




"""
============================
Author:柠檬班-木森
Time:2020/7/16   16:20
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
============================
"""
from setuptools import setup, find_packages

with open("readme.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

setup(
    name='apin',
    version='1.1.4',
    author='MuSen',
    author_email='musen_nmb@qq.com',
    url='https://github.com/musen123/apin',
    description="apin 一个不用写代码的接口自动化测试框架",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["Jinja2==2.10.1",
                      'PyYAML==5.3.1',
                      'requests==2.24.0',
                      'PyMySQL==1.0.2',
                      'rsa==4.7.2',
                      'jsonpath==0.82',
                      'pyasn1==0.4.8',
                      'colorama==0.4.4',
                      'faker==8.11.0'],
    packages=find_packages(),
    package_data={
        "": ["*.html", "*.md", "*.py", '*.json', "*.yaml"],
    },
    # 指定python版本
    python_requires='>=3.6',

    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    # 安装时生成脚本命令行脚本文件
    entry_points={
        "console_scripts": [
            "apin = apin.manage:main",
        ],
    }
)



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import base64
import time
import rsa
from faker import Faker

fk = Faker(locale='zh_CN')


def random_mobile():
    """随机生成手机号"""
    return fk.phone_number()


def random_name():
    """随机生成中文名字"""
    return fk.name()


def random_ssn():
    """随机生成一个省份证号"""
    return fk.ssn()


def random_addr():
    """随机生成一个地址"""
    return fk.address()


def random_city():
    """随机生成一个城市名"""
    return fk.city()


def random_company():
    """随机生成一个公司名"""
    return fk.company()


def random_postcode():
    """随机生成一个邮编"""
    return fk.postcode()


def random_email():
    """随机生成一个邮箱号"""
    return fk.email()


def random_date():
    """随机生成一个日期"""
    return fk.date()


def radom_date_time():
    """随机生成一个时间"""
    return fk.date_time()


def random_ipv4():
    """随机生成一个ipv4的地址"""
    return fk.ipv4()


def get_timestamp():
    """生成当前时间戳"""
    return time.time()


def base64_encode(data: str):
    """base64编码"""
    return base64.b64encode(data.encode('utf-8')).decode('utf-8')


def md5_encrypt(data: str):
    """md5加密"""
    from hashlib import md5
    new_md5 = md5()
    new_md5.update(data.encode('utf-8'))
    return new_md5.hexdigest()


def rsa_encrypt(msg, server_pub):
    """
    rsa加密
    :param msg: 待加密文本
    :param server_pub: 密钥
    :return:
    """
    msg = msg.encode('utf-8')
    pub_key = server_pub.encode("utf-8")
    public_key_obj = rsa.PublicKey.load_pkcs1_openssl_pem(pub_key)  #
    cryto_msg = rsa.encrypt(msg, public_key_obj)  # 生成加密文本
    cipher_base64 = base64.b64encode(cryto_msg)  # 将加密文本转化为 base64 编码
    return cipher_base64.decode()



"""
============================
Project: Apin
Author:柠檬班-木森
Time:2021/7/6 17:23
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
Site: http://www.lemonban.com
Forum: http://testingpai.com 
============================
"""


class DBMysql:
    """mysql databases"""

    def __init__(self, config):
        import pymysql
        config['autocommit'] = True
        self.con = pymysql.connect(**config)
        self.cur = self.con.cursor(pymysql.cursors.DictCursor)

    def execute(self, sql):
        self.cur.execute(sql)
        return self.cur.fetchone()

    def execute_all(self, sql):
        self.cur.execute(sql)
        return self.cur.fetchall()

    def __del__(self):
        self.cur.close()
        self.con.close()


class DBClient:

    def __init__(self, DB):
        for config in DB:
            if not config.get('name'): raise ValueError('数据库配置的name字段不能为空！')
            if config.get('db'):
                setattr(self, config.get('name'), config.get('db'))
            elif config.get('type').lower() == 'mysql' and config.get('config'):
                setattr(self, config.get('name'), DBMysql(config.get('config')))
            else:
                raise ValueError('您传入的数据库配置有误，或者apin目前不支持：{}'.format(config))



# Author:柠檬班-木森
# E-mail:musen_nmb@qq.com
import copy
import re
import json
import os
from numbers import Number
import requests
import jsonpath
from apin.core.dataParser import DataParser
from apin.core.initEvn import ENV, func_tools, DB
from apin.core.basecase import BaseTestCase
from apin.core.logger import CaseLog
from requests_toolbelt.multipart import MultipartEncoder


class CaseData:
    __attrs = ['url', 'method', 'params', 'data', 'json', 'files', 'headers', 'cookies', 'auth', 'timeout',
               'allow_redirects', 'proxies', 'hooks', 'stream', 'verify', 'cert']

    def __init__(self, title="", host="", interface="", extract=None, verification=None, url=None, method=None,
                 params=None, data=None, headers=None, cookies=None, files=None,
                 auth=None, timeout=None, allow_redirects=True, proxies=None,
                 hooks=None, stream=None, verify=None, cert=None, json=None, ):
        """Constructs a :class:`Request <Request>`, prepares it and sends it.
        Returns :class:`Response <Response>` object.

        :param method: method for the new :class:`Request` object.
        :param url: URL for the new :class:`Request` object.
        :param params: (optional) Dictionary or bytes to be sent in the query
            string for the :class:`Request`.
        :param data: (optional) Dictionary, list of tuples, bytes, or file-like
            object to send in the body of the :class:`Request`.
        :param json: (optional) json to send in the body of the
            :class:`Request`.
        :param headers: (optional) Dictionary of HTTP Headers to send with the
            :class:`Request`.
        :param cookies: (optional) Dict or CookieJar object to send with the
            :class:`Request`.
        :param files: (optional) Dictionary of ``'filename': file-like-objects``
            for multipart encoding upload.
        :param auth: (optional) Auth tuple or callable to enable
            Basic/Digest/Custom HTTP Auth.
        :param timeout: (optional) How long to wait for the server to send
            data before giving up, as a float, or a :ref:`(connect timeout,
            read timeout) <timeouts>` tuple.
        :type timeout: float or tuple
        :param allow_redirects: (optional) Set to True by default.
        :type allow_redirects: bool
        :param proxies: (optional) Dictionary mapping protocol or protocol and
            hostname to the URL of the proxy.
        :param stream: (optional) whether to immediately download the response
            content. Defaults to ``False``.
        :param verify: (optional) Either a boolean, in which case it controls whether we verify
            the server's TLS certificate, or a string, in which case it must be a path
            to a CA bundle to use. Defaults to ``True``.
        :param cert: (optional) if String, path to ssl client cert file (.pem).
            If Tuple, ('cert', 'key') pair.
        :rtype: requests.Response
        """
        self.title = title
        self.host = host
        self.interface = interface
        self.verification = verification
        self.extract = extract
        self.url = url
        self.method = method
        self.params = params
        self.data = data
        self.json = json
        self.files = files
        self.headers = headers
        self.cookies = cookies
        self.auth = auth
        self.timeout = timeout
        self.allow_redirects = allow_redirects
        self.proxies = proxies
        self.hooks = hooks
        self.stream = stream
        self.verify = verify
        self.cert = cert
        self.datas = {}

    def data_handle(self, test):
        """请求数据处理"""
        # ------------如果用例没有设置url,method，headers则获取类属性中的------------------
        if not self.host:
            host = getattr(test, 'host', None) or ENV.get('host')
            if host:
                setattr(self, 'host', host)
            else:
                raise ValueError('用例参数host不能为空')
        if not self.interface:
            interface = getattr(test, 'interface', None) or ENV.get('interface')
            if interface:
                setattr(self, 'interface', interface)
            else:
                raise ValueError('用例参数interface不能为空')

        self.url = self.host + self.interface

        if not getattr(self, 'method', None):
            method = getattr(test, 'method', None) or ENV.get('method')
            if method:
                setattr(self, 'method', method)
            else:
                raise ValueError('用例参数method不能为空')

        if not getattr(self, 'headers', None):
            headers = getattr(test, 'headers', None) or ENV.get('headers')
            if headers:
                setattr(self, 'headers', headers)

        # =======处理文件上传===============
        files = self.datas.get('files')
        if self.files:
            if isinstance(self.files, dict):
                file_data = self.files.items()
            else:
                file_data = self.files
            dic = {}
            for name, file_info in file_data:
                # 判断是否时文件上传
                if len(file_info) == 3 and os.path.isfile(file_info[1]):
                    dic[name] = (file_info[0], open(file_info[1], 'rb'), file_info[2])
                else:
                    dic[name] = file_info
            form_data = MultipartEncoder(fields=dic)
            self.headers["Content-Type"] = form_data.content_type
            self.data = form_data
            self.files = None
        else:
            del self.headers["Content-Type"]

        for k, v in self.__dict__.items():
            if k in self.__attrs:
                self.datas[k] = v

    def get(self, attr):
        return getattr(self, attr, None)


class Extract:
    """数据提取"""

    def json_extract(self, response, ext):
        """jsonpath数据提取"""
        value = jsonpath.jsonpath(response.json(), ext)
        value = value[0] if value else ''
        return value

    def re_extract(self, response, ext):
        """正则表达式提取数据提取"""
        value = re.search(ext, response.text)
        value = value.group(1) if value else ''
        return value


class HttpCase(BaseTestCase, Extract, CaseLog):
    env = {}
    host = None
    interface = None
    headers = None
    method = None
    Cases = []
    # 测试结果
    test_result = []
    session = requests.Session()

    def perform(self, case):
        self.__run_log()
        # 前置数据库查询
        db_func = self.__get_db_check(case)
        if db_func:
            self.DBCheck = db_func(self, DB, ENV, self.env)
            try:
                self.debug_log('执行前置sql语句')
                next(self.DBCheck)
            except StopIteration:
                del self.DBCheck
        # 发送http请求
        response = self.http_requests(case)
        # 数据提取
        self.data_extraction(response, case)
        self.__run_log()
        # 响应断言
        self.assert_result(response, case)
        # 前置数据库后置查询和校验
        self.__assert_db()

    def data_extraction(self, response, case):
        """
        数据提取
        :param response: response对象
        :param item: 要提数据的数据，列表嵌套字典
        :return:
        """
        exts = case.get('extract') or getattr(self, 'extract', None)
        if not (isinstance(exts, dict) and exts): return
        self.info_log("从响应结果中开始提取数据")
        self.extras = []
        # 遍历要提取的数据
        for name, ext in exts.items():
            # 判断提取数据的方式
            if len(ext) == 3 and ext[1] == "jsonpath":
                value = self.json_extract(response, ext[2])
            elif len(ext) == 3 and ext[1] == "re":
                value = self.re_extract(response, ext[2])
            else:
                self.error_log("变量{},的提取表达式 :{}格式不对！".format(name, ext))
                self.extras.append((name, ext, '提取失败！'))
                break
            if ext[0] == 'ENV':
                ENV[name] = value
            elif ext[0] == 'env':
                self.env[name] = value
            else:
                self.error_log("错误的变量级别，变量提取表达式中的变量级别只能为ENV，或者env".format(ext[1]))
                continue
            self.extras.append((name, ext, value))
            self.info_log("提取变量：{},提取方式【{}】,提取表达式:{},提取值为:{}".format(name, ext[1], ext[2], value))

    def http_requests(self, case):
        # 发送请求
        # 处理请求数据
        self.__request_hook(case, self.env, ENV, DB)
        case = self.__handle_data(case)
        self.info_log('正在发送请求：')
        response = Request(case, self).request_api()
        self.__response_hook(case, response, self.env, ENV, DB)
        self.response = response
        return response

    def assert_result(self, response, case):
        """断言"""
        self.assert_info = []
        # 获取断言数据
        assert_list = case.get('verification') or getattr(self, 'verification', None)
        # 判断是否需要断言
        if assert_list and isinstance(assert_list, list):
            # 遍历断言的字段
            for item in assert_list:
                # 判断断言数据的类型
                if isinstance(item, list) and len(item) == 3:
                    self.__verification(response, item)
                else:
                    raise ValueError("断言表达式 {} 格式错误:,\n断言表达式必须为如下格式：[断言方式,预期结果,实际结果]".format(item))
        elif assert_list:
            raise ValueError("""{}verification字段格式错误
            verification字段必须为如下格式：[
                [断言方式,预期结果,实际结果]
            ]""".format(assert_list))

    def __verification(self, response, item: list):

        self.info_log('断言表达式:{}'.format(item))
        # 判断断言的方法
        if item[2] == "status_code":
            if item[0] == 'eq':
                actual = response.status_code
                expected = item[1]
                self.info_log('断言http响应状态码是否和预期一致')
                return self.__assert(self.assertEqual, expected, actual, 'eq')
            else:
                self.error_log('http状态码，断言方式必须使用eq')
                raise ValueError('http状态码，断言方式必须使用eq')

        actual = self.__actualDataHandle(response, item[2])
        expected = item[1]
        expected = DataParser.parser_variable(self.env, expected) if expected else expected
        if item[0] == 'eq':
            self.__assert(self.assertEqual, expected, actual, 'eq')
        elif item[0] == 'contains':
            self.__assert(self.assertIn, expected, actual, 'contains')
        else:
            raise ValueError('断言方法有误！断言方式只支持 eq 和 contains')

    def __handle_data(self, case):
        """处理用例数据"""
        if isinstance(case, CaseData):
            data = case
        elif isinstance(case, dict):
            data = CaseData()
            for k, v in case.items():
                setattr(data, k, v)
        else:
            raise TypeError('用例数据只能为dict类型CaseData类型')

        for k, v in data.__dict__.items():
            if k not in ["extract", "verification"]:
                # 解析数据中的函数
                v = DataParser.parser_func(self.env, v)
                # 解析数据中的变量
                v = DataParser.parser_variable(self.env, v)
                setattr(data, k, v)
        data.data_handle(self)
        return data

    def __run_log(self):
        """输出当前环境变量数据的日志"""
        self.l_env = ['{}:{}\n'.format(k, repr(v)) for k, v in self.env.items()]
        self.g_env = ['{}:{}\n'.format(k, repr(v)) for k, v in ENV.items()]
        self.debug_log("全局变量：\n{}".format(''.join(self.g_env)))
        self.debug_log("局部变量：\n{}".format(''.join(self.l_env)))

    def __actualDataHandle(self, response, act):
        """处理实际结果"""
        if isinstance(act, Number):
            return act
        actual_pattern = r"V{{(.+?)}}"
        actual = DataParser.parser_variable(self.env, act) if act else act
        if isinstance(actual, dict):
            for k, v in actual.items():
                res = re.search(actual_pattern, v)
                if res:
                    path = res.group(1)
                    v1 = self.json_extract(response, path)
                    v2 = self.re_extract(response, path)
                    value = v1 if v1 or v1 == 0 else v2
                    actual[k] = value
        elif isinstance(actual, list):
            for k, v in enumerate(copy.deepcopy(actual)):
                res = re.search(actual_pattern, v)
                if res:
                    path = res.group(1)
                    v1 = self.json_extract(response, path)
                    v2 = self.re_extract(response, path)
                    value = v1 if v1 or v1 == 0 else v2
                    actual[k] = value
        else:
            res = re.search(actual_pattern, actual)
            if res:
                path = res.group(1)
                v1 = self.json_extract(response, path)
                v2 = self.re_extract(response, path)
                value = v1 if v1 or v1 == 0 else v2
                actual = value
        return actual

    def __assert(self, assert_method, expected, actual, method):
        """断言"""
        self.info_log("预期结果：{} ".format(expected))
        self.info_log("实际结果：{}".format(actual))
        try:
            assert_method(expected, actual)
        except AssertionError as e:
            self.assert_info.append((repr(expected), repr(actual), 'fail', method))
            self.warning_log('断言未通过')
            raise e
        else:
            self.assert_info.append((repr(expected), repr(actual), 'pass', method))
            self.info_log('断言通过！')

    def __assert_db(self):
        if hasattr(self, 'DBCheck'):
            try:
                self.debug_log('执行后置sql语句')
                next(self.DBCheck)
            except StopIteration as e:
                assert_list = e.value
                if assert_list and isinstance(assert_list, list):
                    self.info_log('开始数据库校验')
                    # 遍历断言的字段
                    for item in assert_list:
                        # 判断断言数据的类型
                        if isinstance(item, list) and len(item) == 3:
                            self.__verification(self.response, item)
                        else:
                            raise ValueError("断言表达式 {} 格式错误:,\n断言表达式必须为如下格式：[断言方式,预期结果,实际结果]".format(item))
                else:
                    raise TypeError("""db_check_hook中返回的数据库校验规则,必须为如下格式：[[断言方式,预期结果,实际结果]]""")

    def __request_hook(self, case, env, ENV, db):
        """请求钩子函数"""
        test = self
        request_shell = case.get('request_hook') or self.get('request_hook')
        if request_shell:
            self.info_log('执行请求前置脚本')
            try:
                exec(request_shell)
            except Exception as e:
                self.error_log('请求前置脚本执行错误:\n{}'.format(e))

    def __response_hook(self, case, response, env, ENV, db):
        test = self
        """响应钩子函数"""
        response_shell = case.get('response_hook') or self.get('response_hook')
        if response_shell:
            self.info_log('执行请求后置脚本')
            try:
                exec(response_shell)
            except Exception as e:
                self.error_log('执行请求后置脚本执行错误:\n{}'.format(e))

    def __get_db_check(self, case):
        hook = self.get('db_check_hook') or case.get('db_check_hook')
        if not hook: return
        # 执行setup_hook方法
        if not isinstance(hook, str):
            raise ValueError('db_check_hook只能传递funcTools中定义的函数名')
        func = getattr(func_tools, hook)
        if not func:
            raise ValueError('函数引用错误：\n{}\n中的函数{}未定义！,'.format(func_tools, hook))
        return func

    @staticmethod
    def __perform_fixture(test, hook_name):
        if not hasattr(test, hook_name):
            return
        hook = getattr(test, hook_name)
        # 执行setup_hook方法
        if not isinstance(hook, str):
            raise ValueError('{}只能传递funcTools中定义的函数名,字符串类型'.format(hook_name))
        func = getattr(func_tools, hook)
        if not func:
            raise ValueError('函数引用错误：\n{}\n中的函数{}未定义！,'.format(func_tools, hook))
        return func

    def setUp(self):
        func = self.__perform_fixture(self, 'setup_hook')
        if func:
            self.info_log('执行用例前置钩子函数')
            func(self, ENV, self.env, DB)

    def tearDown(self):
        func = self.__perform_fixture(self, 'teardown_hook')
        if func:
            self.info_log('执行用例后置钩子函数')
            func(self, ENV, self.env, DB, self.response)

    @classmethod
    def setUpClass(cls):
        func = cls.__perform_fixture(cls, 'setup_class_hook')
        if func:
            cls.log.info('执行测试集前置钩子函数')
            func(ENV, cls.env, DB)

    @classmethod
    def tearDownClass(cls):
        func = cls.__perform_fixture(cls, 'teardown_class_hook')
        if func:
            cls.log.info('执行用测试集置钩子函数')
            func(ENV, cls.env, DB)


class Request:

    def __init__(self, case, test: HttpCase):
        """
        :param case: 用例数据
        :param test: 测试用例
        """
        self.test = test
        self.request_data = case

    def request_api(self):

        # 发送请求
        try:
            self.test.url = self.request_data.datas.get('url')
            self.test.method = self.request_data.datas.get('method')
            response = self.test.session.request(**self.request_data.datas)
        except Exception as e:
            raise ValueError('请求发送失败，错误信息如下：{}'.format(e))
        base_info = "[{}]: {} ".format(self.request_data.method.upper(), self.request_data.url)
        self.test.debug_log(base_info)
        self.test.url = self.request_data.url
        self.test.method = response.request.method
        self.test.status_cede = response.status_code
        self.test.base_info = base_info
        self.test.response_header = response.headers.items()
        self.test.requests_header = response.request.headers.items()
        try:
            response_body = response.json()
            self.test.response_body = json.dumps(response_body, ensure_ascii=False, indent=2)
        except:
            body = response.content
            self.test.response_body = body.decode('utf-8') if body else ''
        try:
            request_body = json.loads(response.request.body.decode('utf-8'))
            self.test.requests_body = json.dumps(request_body, ensure_ascii=False, indent=2)
        except:
            body = response.request.body
            self.test.requests_body = body or ''
        self.requests_log(self.test)
        return response

    def requests_log(self, test):
        requests_log_info = "请求信息如下:"
        requests_log_info += "\nRequest Headers:\n"
        for k, v in getattr(test, 'requests_header'):
            requests_log_info += "      {}:{}\n".format(k, v)
        requests_log_info += "Request body:\n"
        requests_log_info += "{}".format(getattr(test, 'requests_body'))
        response_log_info = "接收后台的响应结果，响应信息如下:"
        response_log_info += "\nResponse Headers:\n"
        for k, v in getattr(test, 'requests_header'):
            response_log_info += "      {}:{}\n".format(k, v)
        response_log_info += "Response body:\n"
        response_log_info += getattr(test, 'response_body')
        self.test.debug_log(requests_log_info)
        self.test.debug_log(response_log_info)



"""
============================
Project: Apin
Author:柠檬班-木森
Time:2021/7/9 16:44
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
Site: http://www.lemonban.com
Forum: http://testingpai.com 
============================
"""
import random,time

def setup_hook1(test, ENV, env, db):
    """
    随机生成一个手机号码，保存为全局变量user_phone
    :param ENV: 全局变量
    :param env: 局部变量
    :return:
    """
    phone = ''
    for i in range(8):
        phone += str(random.randint(0, 9))
    env.user_phone = phone


def get_timestamp_hook(test, ENV, env, db):
    """获取时间戳,保存为全局变量"""
    ENV.timestamp = time.time()

def register_db_check(test, db, ENV, env):
    # 前置sql
    sql = "SELECT count(*) as count FROM futureloan.member WHERE mobile_phone='{}';".format(env.user_mobile)
    s_count = db.qcd.execute(sql)['count']
    yield
    # 后置sql
    e_count = db.qcd.execute(sql)['count']
    # sql校验规则
    return [
        ['eq', s_count + 1, e_count]
    ]



"""
============================
Author:柠檬班-木森
Time:2021/3/1 20:19
E-mail:3247119728@qq.com
Company:湖南零檬信息技术有限公司
=======
"""
# import importlib
#
# func_tools = importlib.import_module('func_tools')
#
# res = getattr(func_tools,"rand_phone")
# print(res())

# import re
#
# s = 'F{func(${{name}},${{age}})}'
#
# res = re.search(r'F{(.*?)\((.*?)\)}', s)
# print(res)
# print(res.group(1))
#
# s_params = res.group(2)
# ss = re.findall(r'\${{.+?}}',s_params )
# print(ss)



